@@ -13,7 +13,7 @@ Modification History:
 ******************/
 
 /************************************
-Copyright (c) 2000                              RIPE NCC
+Copyright (c) 2000,2001,2002                    RIPE NCC
 
 All Rights Reserved
   
@@ -299,6 +299,9 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define ca_get_country	ca_get_string(CA_COUNTRY)
 #define ca_set_country	ca_set_string(CA_COUNTRY)
 
+#define ca_get_authmethod	ca_get_string(CA_AUTHMETHOD)
+#define ca_set_authmethod	ca_set_string(CA_AUTHMETHOD)
+
 #define ca_get_dictionary ca_getDictionary(dictionary, VARS)
 
 #define ca_get_srcname(X)	ca_srchandle2Strelement((X), CA_SRCNAME)
@@ -385,6 +388,15 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define ca_get_ac_decay_halflife ca_get_int(CA_AC_DECAY_HALFLIFE)
 #define ca_set_ac_decay_halflife ca_set_int(CA_AC_DECAY_HALFLIFE)
 
+#define ca_get_ac_save_interval ca_get_int(CA_AC_SAVE_INTERVAL)
+#define ca_set_ac_save_interval ca_set_int(CA_AC_SAVE_INTERVAL)
+
+#define ca_get_ac_auto_save ca_get_boolean(CA_AC_AUTO_SAVE)
+#define ca_set_ac_auto_save ca_set_boolean(CA_AC_AUTO_SAVE)
+
+#define ca_get_ac_load ca_get_boolean(CA_AC_LOAD)
+#define ca_set_ac_load ca_set_boolean(CA_AC_LOAD)
+
 #define ca_get_pw_resp_header	ca_get_string(CA_PW_RESP_HEADER)
 #define ca_set_pw_resp_header	ca_set_string(CA_PW_RESP_HEADER)
 
