@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.18 $
+  $Revision: 1.20 $
 
   Email Parser module (ep) - wrapping functions to parse email,
   calling MM and PA. Header file.
@@ -114,6 +114,7 @@ typedef struct EP_Token {
   EPTokenKeysPtr keys;
   EPTokenPtr next;
   EPTokenPtr prev;
+  GSList     *passwords;
 } EPToken;
 
 typedef struct EP_TokenKeys {
@@ -128,13 +129,9 @@ extern "C" {
 #endif
 
 EP_Mail_DescrPtr EP_ParseMail(   const char *inputFile,
-				 const char *outputPath,
-				 const char *keyRing,
-				 const char *gpgcmd);
+				 const char *outputPath);
 EPNodePtr EP_ParseText(          const char *inputFile,
-		                 const char *outputPath,
-				 const char *keyRing,
-				 const char *gpgcmd);
+		                 const char *outputPath);
 EPNodePtr EP_MIMEParse(          const EPNodePtr p);
 EPNodePtr EP_InitializeRootNode( const char *inputFile );
 EPNodePtr EP_InitializeNode(     const char *inputFile, 
@@ -149,7 +146,7 @@ void EP_MailDescrCleanUp(        const EP_Mail_DescrPtr ptr);
 void EP_BuildFilename(           const EPNodePtr ptr);
 void EP_ShowTree(                const EPNodePtr p);
 EPTokenPtr EP_GetTokens(const EPNodePtr p, const EPTokenPtr prev,
-			EPTokenKeysPtr keysList);
+			EPTokenKeysPtr keysList, GSList *passwords);
 void EP_PrintTokens(EPTokenPtr head);
 void EP_CleanTokens(const EPTokenPtr head);
 
