@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.11 $
+  $Revision: 1.14 $
 
   NT (Notifications) module
 
@@ -41,7 +41,9 @@
 extern int supress_ack_notif;
 extern char * defmail;
 extern int reading_from_mail;
+extern int webupdate;
 extern char * forwlog;
+extern char *netupdclientIP;
 
 /*  Generates a unique file name and returns the full path of the filename 
     for storing notification message.  Creates the file at the same time. 
@@ -51,6 +53,7 @@ char * NT_ntfy_filename_generate( const char * tmpdir, const char * e_mail)
 {
    FILE * ntfy_file;
    char * name;
+   char * replaced_notihdr;
    char * replaced_notimailtxt;
    char * replaced_notinetworktxt;
        
@@ -65,7 +68,14 @@ char * NT_ntfy_filename_generate( const char * tmpdir, const char * e_mail)
      fprintf(stderr, "Can't open notification file for creating, %s", name);
    }
 
-   fprintf(ntfy_file, "To: %s\nFrom: %s\nSubject: Notification of RIPE Database changes\nReply-To: %s\n\n%s\n", e_mail, humailbox, humailbox, notitxt);
+   /* fprintf(ntfy_file, "To: %s\nFrom: %s\nSubject: Notification of RIPE Database changes\nReply-To: %s\n\n%s\n", 
+                     e_mail, humailbox, humailbox, notitxt);*/
+
+   fprintf(ntfy_file, "To: %s\n", e_mail);
+   replaced_notihdr = UP_replace_globals(notihdr);
+   fprintf(ntfy_file, "%s\n\n", replaced_notihdr);
+   free(replaced_notihdr);
+   fprintf(ntfy_file, "\n%s\n", notitxt);
    if (reading_from_mail)
    {
      replaced_notimailtxt = UP_replace_globals(notimailtxt);
@@ -73,7 +83,7 @@ char * NT_ntfy_filename_generate( const char * tmpdir, const char * e_mail)
      free(replaced_notimailtxt);
    }
 
-   if (networkupdate)
+   if (networkupdate || webupdate )
    {
      replaced_notinetworktxt = UP_replace_globals(notinetworktxt);
      fprintf(ntfy_file, "%s\n\n", replaced_notinetworktxt);
@@ -95,7 +105,9 @@ char * NT_forwd_filename_generate( const char * tmpdir, const char * e_mail)
 {
    FILE * forwd_file;
    char * name;
+   char * replaced_fwhdr;
    char * replaced_fwmailtxt;
+   char * replaced_notinetworktxt;
           
    /* allocate space for name.  32 should be enough for PID */
    name = (char*)malloc(strlen(tmpdir) + strlen(e_mail) + strlen("forwd") +36 ); 
@@ -107,13 +119,27 @@ char * NT_forwd_filename_generate( const char * tmpdir, const char * e_mail)
      fprintf(stderr, "Can't open forward file, %s", name);
    }
 
-   fprintf(forwd_file, "To: %s\nFrom: %s\nSubject:  Requested RIPE database object changes \nReply-To: %s\n\n%s\n", e_mail, humailbox, humailbox, fwtxt);
+   /* fprintf(forwd_file, "To: %s\nFrom: %s\nSubject:  Requested RIPE database object changes \nReply-To: %s\n\n%s\n", 
+                      e_mail, humailbox, humailbox, fwtxt);*/
+
+   fprintf(forwd_file, "To: %s\n", e_mail);
+   replaced_fwhdr = UP_replace_globals(fwhdr);
+   fprintf(forwd_file, "%s\n\n", replaced_fwhdr);
+   free(replaced_fwhdr);
+   fprintf(forwd_file, "\n%s\n", fwtxt);
+
    if (reading_from_mail)
    {
      replaced_fwmailtxt = UP_replace_globals(fwmailtxt);
      fprintf(forwd_file, "\n%s\n", replaced_fwmailtxt);
      free(replaced_fwmailtxt);
    }
+   else if (networkupdate || webupdate)
+   {
+     replaced_notinetworktxt = UP_replace_globals(notinetworktxt);
+     fprintf(forwd_file, "\n%s\n", replaced_notinetworktxt);
+     free(replaced_notinetworktxt);
+   }
 
    /* close it */
    fclose(forwd_file);
@@ -439,7 +465,14 @@ void NT_log_ntfy( const char * filename, const char * logfilename)
   /* cut the '\n' at the end */
   time_str[strlen(time_str) - 1] = '\0';
 
-  fprintf(log_file, ">>> time: %s NOTIF <<<\n\n", time_str);
+  if ( networkupdate )
+    fprintf(log_file, ">>> time: %s NETWORKUPDATE NOTIF (%s) <<<\n\n", 
+                time_str, netupdclientIP ? netupdclientIP : "NULL");
+  else if ( webupdate )
+    fprintf(log_file, ">>> time: %s WEB UPDATE NOTIF (%s) <<<\n\n", 
+                time_str, netupdclientIP ? netupdclientIP : "NULL");
+  else
+    fprintf(log_file, ">>> time: %s NOTIF <<<\n\n", time_str);
 
 
   while ( (buf=fgets(buf, 1024, notif_file)) != NULL )
@@ -841,7 +874,10 @@ GList * NT_gather_frwd_addresses(char * object_str)
   for ( attr_item = temp; attr_item != NULL ; attr_item = g_list_next(attr_item) )
   {
     email = rpsl_attr_get_clean_value((rpsl_attr_t *)(attr_item->data));
-printf("NT_gather_frwd_addresses: email [%s]\n", email );
+    if (tracing)
+    {
+      printf("NT_gather_frwd_addresses: email [%s]\n", email );
+    }
     email_list = g_list_append(email_list, email );
   }
   return email_list;
@@ -1161,13 +1197,54 @@ void NT_write_all_frwds(char * old_object_str, char * new_object_str, const char
 void NT_send_forw_creation( const char * filename, const char * to_address, const char * mailercommand)
 {
     char * mail_command_line = NULL;
+    char * supress_file = NULL;
+    FILE * fc_file, * supr_file_hdl;
+    char buf[1024];
 
-    if (to_address != NULL)
+    /* if we are not supressing acks and notifs, send the message */
+    if (!supress_ack_notif)
 	{
-      mail_command_line = (char *)malloc(strlen(mailercommand) + strlen(filename) + 128);
-      sprintf(mail_command_line, "%s %s < %s", mailercommand, to_address, filename);
+      if (to_address != NULL)
+	  {
+        mail_command_line = (char *)malloc(strlen(mailercommand) + strlen(filename) + 128);
+        sprintf(mail_command_line, "%s %s < %s", mailercommand, to_address, filename);
+        system(mail_command_line);
+        free(mail_command_line);
+      }
+    }
+	else
+	{
+      /* if we are supressing acks and notifs, send the message to DEFMAIL  */
+      supress_file = (char *)malloc(strlen(filename) + strlen(".supress") + 2); 
+      sprintf(supress_file, "%s.supress", filename);
+      if (( supr_file_hdl = fopen(supress_file, "w")) == NULL)
+	  {
+        fprintf(stderr, "Can't open supress forward creation file, %s", supress_file);
+      }
+	  else
+	  {
+        fprintf(supr_file_hdl, "From: %s\nTo: %s\nSubject: Supressed forward creation mail\n\n",
+            humailbox, defmail);
+        if (( fc_file = fopen(filename, "r")) == NULL)
+		{
+          fprintf(stderr, "Can't open forward creation file for reading, %s", filename);
+        }
+		else
+		{
+          while (fgets(buf, 1023, fc_file) != NULL)
+		  {
+            fprintf(supr_file_hdl, buf);
+          }
+          fclose(fc_file);
+        }
+      }
+      fclose(supr_file_hdl);
+      mail_command_line = (char *)malloc(strlen(mailercommand) + strlen(defmail) 
+            + strlen(supress_file) + 128);
+      sprintf(mail_command_line, "%s %s < %s", mailercommand, defmail, supress_file);
       system(mail_command_line);
-      free(mail_command_line);
+      unlink(supress_file);
+      free(supress_file);
     }
 }
 
