@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.38 $
+  $Revision: 1.43 $
 
   gpg.c - core of the PA module. Contains functions that are used
   to check the PGP authentication in a message.
@@ -47,13 +47,11 @@
 #include <sys/time.h>
 #include <sys/param.h>
 
-extern char EP_outputPrefix[FILENAME_LENGTH];
-extern char EP_keyRing[FILENAME_LENGTH];
-extern char EP_gpgcmd[FILENAME_LENGTH];
+
 extern int  EP_TreeHeight;
 extern int  EP_Node_ID;
 extern int  EP_Debug;
-extern char *tmpdir;
+
 
 /* static int parseMailBlock_nMsg;
    static int parseRecursionLevel; */
@@ -65,6 +63,11 @@ extern time_t nfslock(char *path, char *namelock, int max_age, int notify);
 extern int nfsunlock(char *path, char *namelock, int max_age, time_t birth);
 
 
+static char* outputPrefix = NULL;
+static char* keyRing = NULL;
+static char* gpgCmd = NULL;
+static char *tmpdir = NULL;
+
 static void VerifySignAndExplodeFile(EPNodePtr ptr);
 static void GetKeyID(struct ImportKeyObject *iKO);
 
@@ -75,6 +78,78 @@ static void GetKeyID(struct ImportKeyObject *iKO);
  *
  **************************************/
 
+/*++++++++++++++++++++++++++++
+
+Copies a string with memory allocation care.
+
+char* dest The destination string
+char* orig The origin string
+
+
+++++++++++++++++++++++++++++*/
+
+void pa_strcpy(char** dest, char* orig) {
+  if (*dest) {
+    free(*dest);
+  }
+
+  *dest = malloc(strlen(orig) + 1);
+  strcpy(*dest, orig);
+}
+
+/*++++++++++++++++++++++++++++
+
+Set the GPG temporary directory.
+
+char* dir The directory
+
+
+This memory allocated is never freed, maybe a PA_end function?
+
+++++++++++++++++++++++++++++*/
+
+void PA_SetTmpDir(char* dir) {
+  pa_strcpy(&tmpdir, dir);
+}
+
+/*++++++++++++++++++++++++++++
+
+Set the GPG command line.
+
+char* path The path including command name
+
+
+This memory allocated is never freed, maybe a PA_end function?
+
+++++++++++++++++++++++++++++*/
+
+void PA_SetGPGCmd(char* path) {
+  pa_strcpy(&gpgCmd, path);
+}
+
+/*++++++++++++++++++++++++++++
+
+Set the path to the Keyring.
+
+char* path The path including file name
+
+++++++++++++++++++++++++++++*/
+
+void PA_SetKeyRing(char* path) {
+  pa_strcpy(&keyRing, path);
+}
+
+/*++++++++++++++++++++++++++++
+
+Set the path to the output file.
+
+char* path The path including file name
+
+++++++++++++++++++++++++++++*/
+
+void PA_SetOutputPrefix(char* path) {
+  pa_strcpy(&outputPrefix, path);
+}
 
 /*++++++++++++++++++++++++++++
 
@@ -100,12 +175,13 @@ void PA_VerifySignature(struct VerifySignObject *vSO) {
   int fIn,fOut;
   char tmpFileName[100],lfcrStr[10],strIn[10];
   char prevChar;
+  FILE *mystdin;
 
   vSO->type = vSO_Type_Signed;
 
   strcpy(Args0, "--no-secmem-warning");
   strcpy(Args1, "--keyring");
-  strcpy(Args2, vSO->keyRing);
+  strcpy(Args2, keyRing);
   strcpy(Args3, "-o");
   if (!strcmp(vSO->iSigFilename, "")) {
     strcpy(Args4, vSO->oStream);
@@ -162,7 +238,7 @@ void PA_VerifySignature(struct VerifySignObject *vSO) {
   gpg_in_fd = INPUT_FD;
   out_fd = OUTPUT_FD;
   err_fd = ERROR_FD;
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 )
     {
       ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
@@ -191,8 +267,9 @@ void PA_VerifySignature(struct VerifySignObject *vSO) {
 
 
   /* Parsing gpg output */
+  mystdin = fdopen(0, "r");
   vSO->isValid = vSO_KO;
-  while (fgets (txt, LINE_LENGTH - 1, stdin) != NULL)
+  while (fgets (txt, LINE_LENGTH - 1, mystdin) != NULL)
     {
       /* 	printf ( "GPG output : %s\n", txt );   */
       if (strstr(txt, "Good signature") != NULL)
@@ -247,6 +324,7 @@ void PA_Decrypt(struct ReadCryptedObject *rDO) {
   int status;
   char txt[LINE_LENGTH];
   int childRC;
+  FILE *mystdin;
 
   strcpy(Args0, "--no-tty");
   strcpy(Args1, "--no-secmem-warning");
@@ -268,7 +346,7 @@ void PA_Decrypt(struct ReadCryptedObject *rDO) {
   gpg_in_fd = INPUT_FD;
   out_fd = OUTPUT_FD;
   err_fd = ERROR_FD;
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 )
     {
       ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
@@ -295,7 +373,8 @@ void PA_Decrypt(struct ReadCryptedObject *rDO) {
 
 
   /* Parsing gpg output */
-  while (fgets (txt, STRING_LENGTH - 1, stdin) != NULL)
+  mystdin = fdopen(0, "r");
+  while (fgets (txt, STRING_LENGTH - 1, mystdin) != NULL)
     {
       
     }
@@ -368,7 +447,7 @@ void PA_ImportKey(struct ImportKeyObject *iKO) {
   
   lockBirthDate = nfslock(keyRingPath, (char*)lockFilename, 0, 0);
 
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
     ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
     exit(1);
@@ -386,12 +465,17 @@ void PA_ImportKey(struct ImportKeyObject *iKO) {
   if (WIFEXITED(status) == 0)
     {
       ER_perror(FAC_PA, PA_CHST, "%d - %s", status, ERRSTRING);
-    } else {
+    } 
+    else 
+    {
       /* Child exited, checking return code */
       childRC = (status & 0xF00) >> 8;
-      if (childRC == 1) {
-	ER_perror(FAC_PA, PA_CHRC, "%d", childRC);
-	exit(1);
+      if (tracing)
+        printf("TRACING: PA_ImportKey child return code [%d]\n", childRC);
+      if (childRC == 1) 
+      {
+        ER_perror(FAC_PA, PA_CHRC, "%d", childRC);
+        /*  exit(1);  */
       }
     }
 
@@ -404,38 +488,48 @@ void PA_ImportKey(struct ImportKeyObject *iKO) {
   iKO->rc = iKO_GENERALFAILURE;
   key_count = 0;
   while (fgets (txt, LINE_LENGTH - 1, mystdin) != NULL)
+  {
+    /* printf ( "GPG output : %s\n", txt );      */
+
+    if ((keyStr = strstr(txt, "imported")) != NULL)
     {
-      /*          printf ( "GPG output : %s\n", txt );      */
-
-      if ((keyStr = strstr(txt, "imported")) != NULL) {
-	iKO->rc = iKO_OK;
-      }
-
-      if ((keyStr = strstr(txt, "CRC error")) != NULL) {
-	iKO->rc = iKO_CRC_ERROR;
-      }
-
-      if ((keyStr = strstr(txt, "no valid OpenPGP")) != NULL) {
-	iKO->rc = iKO_NO_OPENPGP_DATA;
-      }
-
-      if (((keyStr = strstr(txt, "unchanged")) != NULL) || 
-	  ((keyStr = strstr(txt, "not changed")) != NULL)) {
-	iKO->rc = iKO_UNCHANGED;
-      }
-
-      if ((keyStr = strstr(txt, "key")) != NULL) {
-	 keyStr += 4;
-	 sscanf(keyStr, "%8X\n", &iKO->keyID); 
-   key_count++;
-   /* and put the key ID into the keys list (if it was new to the keyring)*/
-   if(strstr(txt, "imported") != NULL){
-     key_list = g_list_append(key_list, GINT_TO_POINTER(iKO->keyID));
-   }
-
-      } 
+	  iKO->rc = iKO_OK;
     }
 
+    if ((keyStr = strstr(txt, "CRC error")) != NULL)
+    {
+	  iKO->rc = iKO_CRC_ERROR;
+    }
+
+    if ((keyStr = strstr(txt, "no valid OpenPGP")) != NULL)
+    {
+	  iKO->rc = iKO_NO_OPENPGP_DATA;
+    }
+
+    if ((keyStr = strstr(txt, "invalid armor header")) != NULL)
+    {
+	  iKO->rc = iKO_INVALID_ARMOR_HEADER;
+    }
+
+    if (((keyStr = strstr(txt, "unchanged")) != NULL) || 
+	  ((keyStr = strstr(txt, "not changed")) != NULL))
+    {
+	  iKO->rc = iKO_UNCHANGED;
+    }
+
+    if ((keyStr = strstr(txt, "key")) != NULL)
+    {
+	  keyStr += 4;
+	  sscanf(keyStr, "%8X\n", &iKO->keyID); 
+      key_count++;
+      /* and put the key ID into the keys list (if it was new to the keyring)*/
+      if(strstr(txt, "imported") != NULL)
+      {
+        key_list = g_list_append(key_list, GINT_TO_POINTER(iKO->keyID));
+      }
+    } 
+  }
+
   if (sd1[0] != 0)  close ( sd1[0] ); 
 
   if(key_count > 1){/* if there were more than one keys imported */
@@ -525,7 +619,7 @@ void PA_RemoveKey(struct ImportKeyObject *iKO) {
   
     lockBirthDate = nfslock(keyRingPath, (char*)lockFilename, 0, 0);
 
-    if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+    if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 				&gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
       ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
       exit(1);
@@ -542,33 +636,36 @@ void PA_RemoveKey(struct ImportKeyObject *iKO) {
     nfsunlock(keyRingPath, (char*)lockFilename, 0, lockBirthDate);
 
     if (WIFEXITED(status) == 0)
-      {
+    {
 	ER_perror(FAC_PA, PA_CHST, "%d - %s", status, ERRSTRING);
 	exit(1);
-      }  else {
-	/* Child exited, checking return code */
-	childRC = (status & 0xF00) >> 8;
-	if (childRC == 1) {
-	  ER_perror(FAC_PA, PA_CHRC, "%d", childRC);
-	  exit(1);
-	}
-      }
+    }
+    else
+    {
+	  /* Child exited, checking return code */
+	  childRC = (status & 0xF00) >> 8;
+	  if (childRC == 1)
+      {
+	    ER_perror(FAC_PA, PA_CHRC, "%d", childRC);
+	  }
+    }
 
 
     mystdin = fdopen(0, "r");
     iKO->rc = iKO_OK;
     while (fgets (txt, LINE_LENGTH - 1, mystdin) != NULL)
+    {
+	  /* printf ( "GPG output : %s\n", txt );        */
+
+	  if ((keyStr = strstr(txt, "delete key failed")) != NULL)
       {
-	/* printf ( "GPG output : %s\n", txt );        */
-
-	if ((keyStr = strstr(txt, "delete key failed")) != NULL) {
-	  iKO->rc = iKO_GENERALFAILURE;
-	}
-	if ((keyStr = strstr(txt, "there is a secret key for this public key")) != NULL) {
-	  iKO->rc = iKO_SECRET_KEY_PRESENT;
-	}
-
-      }
+	    iKO->rc = iKO_GENERALFAILURE;
+	  }
+	  if ((keyStr = strstr(txt, "there is a secret key for this public key")) != NULL)
+      {
+	    iKO->rc = iKO_SECRET_KEY_PRESENT;
+	  }
+    }
 
     if (sd1[0] != 0)  close ( sd1[0] ); 
   }
@@ -639,7 +736,7 @@ void PA_RemoveKey_withKeyID(struct ImportKeyObject *iKO) {
     lockBirthDate = nfslock(keyRingPath, (char*)lockFilename, 0, 0);
 
 
-    if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+    if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 				&gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
       ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
       exit(1);
@@ -664,7 +761,6 @@ void PA_RemoveKey_withKeyID(struct ImportKeyObject *iKO) {
 	childRC = (status & 0xF00) >> 8;
 	if (childRC == 1) {
 	  ER_perror(FAC_PA, PA_CHRC, "%d", childRC);
-	  exit(1);
 	}
       }
 
@@ -770,14 +866,14 @@ EPNodePtr PA_ParseMessage(EPNodePtr ptr) {
 	    
 	    /* Called form EP_ParseMail or EP_PArseText ? */
 	    
-	    if (strstr(EP_outputPrefix, "EPMtmp") != NULL)
+	    if (strstr(outputPrefix, "EPMtmp") != NULL)
 	      {
 		ER_dbg_va (FAC_PA, ASP_PA_GEN, "Found prefix and suffix; calling EP_MIMEParse...");
 		EP_MIMEParse(p);
 	      }
-	    else
-	      PA_ParseMessage(p);
-
+	    else {
+              PA_ParseMessage(p);
+            }
 	    prev = p;
 	  } else {
 	    /* Wrong PGP delimiters order. */
@@ -823,13 +919,14 @@ EPNodePtr PA_ParseMessage(EPNodePtr ptr) {
 	    if (found_prefix || (prev != p)) {
 	      if (prev->MIMEContentType == -1)
 		{
-		  if (strstr(EP_outputPrefix, "EPMtmp") != NULL)
+		  if (strstr(outputPrefix, "EPMtmp") != NULL)
 		    {
 		      ER_dbg_va (FAC_PA, ASP_PA_GEN, "PA_ParseMessage: sending to EP_MIMEParse");
 		      EP_MIMEParse(p);
 		    }
-		  else
+		  else {
 		    PA_ParseMessage(p);
+                  }
 		}
 	    
 	      prev = p;
@@ -900,7 +997,7 @@ void GetFingerPrint(struct ImportKeyObject *iKO) {
 
   /* create lock file filenames for NFS */
 
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
     ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
     exit(1);
@@ -989,7 +1086,7 @@ void GetKeyOwner(struct ImportKeyObject *iKO) {
 
   /* create lock file filenames for NFS */
 
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
     ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
     exit(1);
@@ -1036,7 +1133,6 @@ void GetKeyOwner(struct ImportKeyObject *iKO) {
 
 
 
-
 /*++++++++++++++++++++++++++++
 
 Verify the PGP signature and extract the signed part in a file.
@@ -1058,12 +1154,13 @@ void VerifySignAndExplodeFile(EPNodePtr ptr) {
   char oFile[FILENAME_LENGTH];
   char *keyStr;
   /* int childRC; */
+  FILE *mystdin;
 
   sprintf(oFile, "%s.%d.exp", ptr->file, ptr->nodeID);
 
   strcpy(Args0, "--no-secmem-warning");
   strcpy(Args1, "--keyring");
-  strcpy(Args2, EP_keyRing);
+  strcpy(Args2, keyRing);
   strcpy(Args3, "-o");
   strcpy(Args4, oFile);
   strcpy(Args5, "-d");
@@ -1081,36 +1178,29 @@ void VerifySignAndExplodeFile(EPNodePtr ptr) {
   gpg_in_fd = INPUT_FD;
   out_fd = OUTPUT_FD;
   err_fd = ERROR_FD;
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
-			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 )
-    {
+
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
+			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
       ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
       exit(1);
-    }
+  }
   
-  if (waitpid (gpg_pid, &status, 0) < 0)
-    {
-      ER_perror(FAC_PA, PA_REAP, "gpg process: %s", ERRSTRING);
-      exit(1);
-    }
-  if (WIFEXITED(status) == 0)
-    {
-      ER_perror(FAC_PA, PA_CHST, "%d - %s", status, ERRSTRING);
-      exit(1);
-    } else {
-      /* Child exited, checking return code */
-      /* childRC = (status & 0xF00) >> 8;
-      if (childRC == 1) {
-	fprintf (stderr, "Fatal: gpg child return code: %d\n", childRC);
-	printf ("gpg failure\n");
-	exit(1);
-	} */
-    }
+  if (waitpid (gpg_pid, &status, 0) < 0) {
+    ER_perror(FAC_PA, PA_REAP, "gpg process: %s", ERRSTRING);
+    exit(1);
+  }
+  if (WIFEXITED(status) == 0) {
+    ER_perror(FAC_PA, PA_CHST, "%d - %s", status, ERRSTRING);
+    exit(1);
+  }
+  else {
+  }
 
 
   /* Parsing gpg output */
+  mystdin = fdopen(0, "r");
   ptr->isValidPGPSignature = vS_KO;
-  while (fgets (txt, LINE_LENGTH - 1, stdin) != NULL)
+  while (fgets (txt, LINE_LENGTH - 1, mystdin) != NULL)
     {
       /* 	printf ( "GPG output : %s\n", txt );   */
       if (strstr(txt, "Good signature") != NULL)
@@ -1131,6 +1221,13 @@ void VerifySignAndExplodeFile(EPNodePtr ptr) {
       }
     }
     
+  if (ptr->isValidPGPSignature == vS_NO_OPENPGP_DATA) {
+    FILE* f;
+    f= fopen(oFile, "w");
+    fprintf(f, " \n");
+    fclose(f);
+  }
+
   unlink(ptr->file);
   UT_free(ptr->file);
   ptr->file = UT_strdup(oFile);
@@ -1197,7 +1294,7 @@ void GetKeyID(struct ImportKeyObject *iKO) {
   
   lockBirthDate = nfslock(keyRingPath, (char*)lockFilename, 0, 0);
 
-  if ( ( gpg_pid = spawn_job (EP_gpgcmd, strArgs,
+  if ( ( gpg_pid = spawn_job (gpgCmd, strArgs,
 			      &gpg_in_fd, &out_fd, &err_fd) ) < 0 ) {
     ER_perror(FAC_PA, PA_CANTSPWN, "gpg");
     exit(1);
