@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.94 $
+  $Revision: 1.96 $
 
   Query instructions (qi).  This is where the queries are executed.
 
@@ -37,6 +37,11 @@
 #include <string.h>
 #include <glib.h>
 
+/* maximum number of names to query for */
+/*   MySQL only allows 31 tables in a single query, so this limits us */
+/*   As of 2002-03-04 the most names in a person object are 14 */
+#define MAX_NAMES 30
+
 /* the TEMPORARY table is only available in 3.23 or later MySQL */
 #if MYSQL_VERSION_ID >= 32300
 #define USE_TEMPORARY_TABLE 
@@ -208,7 +213,7 @@ static void create_name_query(GString *query_str, const char *sql_query, const c
     g_string_sprintfa(from_clause, "names N%.2d", 0);
     g_string_sprintfa(where_clause, "N%.2d.name=\"%s\"", 0, words[0]);
 
-    for (i=1; words[i] != NULL; i++) {
+    for (i=1; (i < MAX_NAMES) && (words[i] != NULL); i++) {
       g_string_sprintfa(from_clause, ", names N%.2d", i);
       g_string_sprintfa(where_clause, 
           " AND N%.2d.name=\"%s\" AND N00.object_id = N%.2d.object_id", 
@@ -891,30 +896,26 @@ mnt_irt_filter (sk_conn_st *condat,
     new_datlist = NULL;
 
     /* search for node with "mnt-irt:" attribute */
-    p = g_list_last(*datlist);
-    while ((new_datlist == NULL) && (p != NULL)) {
+    for (p=g_list_last(*datlist); p != NULL; p=g_list_previous(p)) {
         /* grab the data for this node */
         rx_data = (rx_datcpy_t *)p->data;
         object_id = rx_data->leafcpy.data_key;
 
         /* see if this is the node we are looking for */
         if (rx_node_has_mnt_irt(condat, sql_connection, object_id)) {
-            /* use this entry */
+            /* use this entry, and remove from old list */
             new_datlist = g_list_append(NULL, rx_data);
+            *datlist = g_list_remove_link(*datlist, p);
+            g_list_free_1(p);
+            break;
         } 
-        else {
-            /* free memory */
-            UT_free(rx_data->leafcpy.data_ptr);
-        }
-
-        p = g_list_previous(p);
     }
 
     /* free our old datlist */
-    while (p != NULL) {
+    for (p=*datlist; p != NULL; p = g_list_next(p)) {
         rx_data = (rx_datcpy_t *)p->data;
         UT_free(rx_data->leafcpy.data_ptr);
-        p = g_list_previous(p);
+        p = g_list_next(p);
     }
     wr_clear_list(datlist);
 
