@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.20 $
+  $Revision: 1.22 $
 
   Socket module - routines facilitating calls to socket library.
 
@@ -143,7 +143,12 @@ int  SK_getsock(int socket_type, unsigned h_port, int backlog,
   setsockopt(listening_socket, SOL_SOCKET, SO_REUSEADDR, (void *)&reuse_addr, sizeof(reuse_addr));
 
   if (bind(listening_socket, (struct sockaddr *) &address, sizeof(address)) < 0) {
-    perror("bind");
+    fprintf(stderr, 
+        "error %d with bind() to %s:%d; %s\n",
+        errno,
+        inet_ntoa(address.sin_addr),
+        h_port,
+        strerror(errno));
     close(listening_socket);
     exit(EXIT_FAILURE);
   }
@@ -320,6 +325,12 @@ er_ret_t SK_connect(int  *sock, char *hostname, unsigned int port, unsigned int
 }
 
 
+/* XXX: deprecated SK_read(), SK_gets(), and SK_getc(), since these 
+        functions are unused, and do not work with the buffering added
+        to SK_cd_gets() 
+        - Shane, 2002-05-13 */
+
+#if 0
 /* SK_read() */
 /*++++++++++++++++++++++++++++++++++++++
 
@@ -357,6 +368,7 @@ int SK_read(int sockfd, char *buf, size_t count) {
   return count;
 
 } /* SK_read() */
+#endif /* 0 */
 
 
 /* SK_write() */
@@ -458,6 +470,7 @@ SK_write(int sockfd,
 } /* SK_write() */
 
 
+#if 0
 /* SK_gets() */
 /*++++++++++++++++++++++++++++++++++++++
 
@@ -524,6 +537,7 @@ int SK_gets(int  sockfd, char *str, size_t count) {
   return total_count;
 
 } /* SK_gets() */
+#endif /* 0 */
 
 
 /* SK_puts() */
@@ -576,6 +590,7 @@ SK_putc(int sockfd, char ch, const struct timeval *timeout)
   return count_sent;
 }/* SK_putc() */
 
+#if 0
 /*++++++++++++++++++++++++++++++++++++++
 
    This function reads a single character from a socket.
@@ -593,6 +608,7 @@ int SK_getc(int  sockfd) {
     return ch;
   }
 }/* SK_getc() */
+#endif /* 0 */
 
 /* SK_getpeername() */
 /*++++++++++++++++++++++++++++++++++++++
