@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.62 $
+  $Revision: 1.63 $
 
   Example code: A server for a client to connect to.
 
@@ -591,10 +591,10 @@ int SV_start(char *pidfile) {
   /* Initialise the access control list. */
   AC_build();
   AC_acc_load();
-  /* explicitly start the decay thread */
+  AC_persistence_init();
+  /* explicitly start the decay & persistence threads */
   TH_create((void *(*)(void *))AC_decay, NULL);
-
-
+  TH_create((void *(*)(void *))AC_persistence_daemon, NULL);
   
   /* Get port information for each service */
   whois_port  = ca_get_svwhois_port;
