@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.34 $
+  $Revision: 1.41 $
 
   Wrapper for NRTM client
 
@@ -11,7 +11,7 @@
   Modification History:
         andrei (17/01/2000) Created.
   ******************/ /******************
-  Copyright (c) 2000                              RIPE NCC
+  Copyright (c) 2000,2001,2002                      RIPE NCC
  
   All Rights Reserved
   
@@ -30,34 +30,26 @@
   AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ***************************************/
+
+#include "rip.h"
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <fcntl.h>
 #include <signal.h>
-/*#include <stream.h>*/
+#include <sys/poll.h>
 
 
-#include "ud.h"
-#include "ud_int.h"
-
-#include "constants.h"
-
-#include "er_macro.h"
-#include "er_paths.h"
-
-#include "server.h"
-#include "protocol_mirror.h"
-#include "ta.h"
-
-#include "iproutines.h"
-#include "aa.h"
-
 /* here we store sockets for update threads */
 /* they are from SV module */
 extern int SV_update_sock[];
 
+/* here is the shutdown pipe for update threads */
+/* also from SV module (apologies for breaking encapsulation) */
+extern int SV_shutdown_recv_fd;
+
 /* Response time to swtching updates on and off */
 #define TIMEOUT 60 
 
@@ -94,23 +86,32 @@ int sockfd;
 struct hostent *hptr;
 struct sockaddr_in serv_addr;
 struct in_addr *paddr;
-char line_buff[STR_XXL];
+char host_info[STR_XXL];
+GString *line_buff;
 int fd;
 int ret;
 struct hostent result;
 int error;
 int network;
 
+int so_keepalive =1;
 
 /* fprintf(stderr, "Making connection to NRTM server ...\n");*/
  if ((sockfd=socket(AF_INET, SOCK_STREAM, 0))==-1){
    ER_perror(FAC_UD, UD_FS, "cannot create socket");
    return(-1);
- }  
+ } 
+
+/* set keepalive option so we can reconnect in case there was no connectivity */
+/* with primary. See bug 100 for details. Default timeout is 2hours           */
+ if ((error=setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE, &so_keepalive, sizeof(so_keepalive))) == -1){
+   ER_perror(FAC_UD, UD_FS, "cannot set KEEPALIVE socket option");
+   return(-1);
+ }
 #ifdef __linux__
- if(gethostbyname_r(nrtm->server,  &result, line_buff, sizeof(line_buff), &hptr, &error)<0)  hptr=NULL;
+ if(gethostbyname_r(nrtm->server,  &result, host_info, sizeof(host_info), &hptr, &error)<0)  hptr=NULL;
 #else/* default is Solaris implementation */
- hptr=gethostbyname_r(nrtm->server,  &result, line_buff, sizeof(line_buff), &error);
+ hptr=gethostbyname_r(nrtm->server,  &result, host_info, sizeof(host_info), &error);
 #endif
 
  /* Check if it is a network stream or a file */
@@ -122,20 +123,22 @@ int network;
    memcpy(&serv_addr.sin_addr, paddr, sizeof(struct in_addr));
 /*   fprintf(stderr,"Trying %s port %d\n", inet_ntoa(serv_addr.sin_addr), nrtm->port);*/
    if(connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr))==-1) {
-     ER_perror(FAC_UD, UD_FS, "cannot connect"); 
+     ER_perror(FAC_UD, UD_FS, "cannot connect, errno:%d", errno); 
      close(sockfd);
      return(-1);
    }  
 /*   fprintf(stderr, "Sending Invitation\n"); */
    
    /* Request all available serials (upto LAST), or SBUNCH of them */
+   line_buff = g_string_sized_new(STR_L);
    if(upto_last==1)
-      sprintf(line_buff, "-g %s:%d:%ld-LAST\n", source, nrtm->version, nrtm->current_serial+1);
+      g_string_sprintf(line_buff, "-g %s:%d:%ld-LAST\n", source, nrtm->version, nrtm->current_serial+1);
    else if(upto_last==-1) /* persistent mirroring */
-      sprintf(line_buff, "-k -g %s:%d:%ld-LAST\n", source, nrtm->version, nrtm->current_serial+1);
+      g_string_sprintf(line_buff, "-k -g %s:%d:%ld-LAST\n", source, nrtm->version, nrtm->current_serial+1);
    else
-      sprintf(line_buff, "-g %s:%d:%ld-%ld\n", source, nrtm->version, nrtm->current_serial+1, nrtm->current_serial+SBUNCH);   
-   ret=SK_write(sockfd, line_buff, strlen(line_buff), NULL, NULL );
+      g_string_sprintf(line_buff, "-g %s:%d:%ld-%ld\n", source, nrtm->version, nrtm->current_serial+1, nrtm->current_serial+SBUNCH);   
+   ret=SK_write(sockfd, line_buff->str, line_buff->len, NULL, NULL );
+   g_string_free(line_buff, TRUE);
    if(ret != 1) { 
 	   ER_perror(FAC_UD, UD_FS, "cannot write");
 	   close(sockfd);
@@ -189,7 +192,7 @@ int do_server;
 int nrtm_fd;
 int num_ok;
 int upto_last;
-char ta_activity[STR_M];
+GString *ta_activity;
 ca_dbSource_t *source_hdl = ca_get_SourceHandleByPosition(source);
 char *db_host, *db_name, *db_user, *db_passwd;
 int db_port;
@@ -199,21 +202,26 @@ char *source_name = ca_get_srcname(source_hdl);
   { /* set up the lohgging path */
    int res;
    char *er_ud_def = ca_get_er_ud_def; /* something like 'RIPUPDLOG basename' */
-   char er_def[256];
+   GString *er_def;
    char *erret = NULL;
 
-   sprintf(er_def, "%s %s", er_ud_def, source_name);
-   fprintf(stderr, "[%s]\n", er_def);
-   if( (res = ER_macro_spec(er_def, &erret)) != 0 ) {
+   er_def = g_string_sized_new(256);
+   g_string_sprintf(er_def, "%s %s", er_ud_def, source_name);
+   fprintf(stderr, "[%s]\n", er_def->str);
+   if( (res = ER_macro_spec(er_def->str, &erret)) != 0 ) {
         fputs(erret, stderr);
         die;
         /* or some other error handling */
    }     
-   free(erret); /* the response is allocated and must be freed */
-   free(er_ud_def);
+   UT_free(erret); /* the response is allocated and must be freed */
+   g_string_free(er_def, TRUE);
+   UT_free(er_ud_def);
   }  
+
+  /* load the dictionary */
+  rpsl_load_dictionary(RPSL_DICT_CORE);
          
-  nrtm=calloc(1, sizeof(struct _nrtm));
+  nrtm=UT_calloc(1, sizeof(struct _nrtm));
   if(nrtm==NULL) {
 	  ER_perror(FAC_UD, UD_MEM, "cannot allocate memory");
 	  die;
@@ -316,8 +324,10 @@ char *source_name = ca_get_srcname(source_hdl);
    
     /* make a record for thread accounting */
     TA_add(nrtm_fd, "nrtm_clnt");
-    sprintf(ta_activity,"[%s]%ld->", source_name, nrtm->current_serial);
-    TA_setactivity(ta_activity);
+    ta_activity = g_string_sized_new(STR_M);
+    g_string_sprintf(ta_activity, "[%s]%ld->", 
+                     source_name, nrtm->current_serial);
+    TA_setactivity(ta_activity->str);
    
    
    ud_stream.condat.sock = nrtm_fd;
@@ -343,8 +353,10 @@ char *source_name = ca_get_srcname(source_hdl);
    ER_inf_va(FAC_UD, ASP_UD_OBJ, "%s forwarded to serial:%ld", UD_TAG, (nrtm->current_serial+num_ok));
    
    /* set activity for thread record */
-   sprintf(ta_activity,"[%s]->%ld", source_name, (nrtm->current_serial+num_ok));
-   TA_setactivity(ta_activity);
+   g_string_sprintf(ta_activity, "[%s]->%ld", 
+                    source_name, (nrtm->current_serial+num_ok));
+   TA_setactivity(ta_activity->str);
+   g_string_free(ta_activity, TRUE);
 
 /* if we are NOT in persistent mode */
    if(!IS_PERSIST_MIRR(ud_stream.ud_mode)) {
@@ -367,19 +379,19 @@ char *source_name = ca_get_srcname(source_hdl);
  } while((do_server=CO_get_do_server()));  /* main cycle */
 
 /*   fclose(ud_stream.log.logfile);*/
-   free(source_name);
+   UT_free(source_name);
 /* free data associated with nrtm structure */         
  if(nrtm) {
-   free(nrtm->server);
-   free(nrtm);
+   UT_free(nrtm->server);
+   UT_free(nrtm);
  }
  
  /* That's all. Close connection to the DB */ 
  SQ_close_connection(ud_stream.db_connection);
- free(db_host);
- free(db_name);
- free(db_user);
- free(db_passwd);
+ UT_free(db_host);
+ UT_free(db_name);
+ UT_free(db_user);
+ UT_free(db_passwd);
 
  ER_dbg_va(FAC_UD, ASP_UD_UPDLOG, "%s NRTM client stopped", UD_TAG); 
 } /* UD_do_nrtm() */
@@ -408,6 +420,7 @@ char *db_host, *db_name, *db_user, *db_passwd;
 int db_port;
 ip_addr_t address;
 char *source_name;
+struct pollfd pfd[2];
 
    source_name = ca_get_srcname(source_hdl);  
 
@@ -415,20 +428,26 @@ char *source_name;
    /* get source we are going to update */
    int res;
    char *er_ud_def = ca_get_er_ud_def; /* something like 'RIPUPDLOG basename' */
-   char er_def[256];
+   GString *er_def;
    char *erret = NULL;
    
 
-   sprintf(er_def, "%s %s", er_ud_def, source_name);
-   if( (res = ER_macro_spec(er_def, &erret)) != 0 ) {
+   er_def = g_string_sized_new(256);
+   g_string_sprintf(er_def, "%s %s", er_ud_def, source_name);
+   if( (res = ER_macro_spec(er_def->str, &erret)) != 0 ) {
         fputs(erret, stderr);
         die;
         /* or some other error handling */
    }     
-   free(erret); /* the response is allocated and must be freed */
-   free(er_ud_def);
+   UT_free(erret); /* the response is allocated and must be freed */
+   g_string_free(er_def, TRUE);
+   UT_free(er_ud_def);
   } 
 
+  /* load the dictionary */
+    rpsl_load_dictionary(RPSL_DICT_CORE);
+    
+
 /* get mode of operation: protected/unprotected (dummy) */
   memset(&ud_stream, 0, sizeof(ud_stream));
   ud_stream.source_hdl=source_hdl;
@@ -464,6 +483,12 @@ char *source_name;
   ud_stream.num_skip=0;
   ud_stream.load_pass=0;
   ud_stream.nrtm=NULL;
+
+  /* set up poll structure */
+  pfd[0].fd = listening_socket;
+  pfd[0].events = POLLIN;
+  pfd[1].fd = SV_shutdown_recv_fd;
+  pfd[1].events = POLLIN;
  
 /*+++ main cycle +++*/
 
@@ -472,6 +497,16 @@ do { /* be alive while do_server is 1. do_server is turned off by SIGINT */
    /* make a record for thread accounting */
    TA_add(listening_socket, "update");
    TA_setactivity("waiting");
+
+   /* check for input */
+   if (poll(pfd, 2, -1) == -1) {
+        ER_perror(FAC_SV, 1, "UD_do_updates got error %d on poll; %s",
+                  errno, strerror(errno));
+        continue;
+   }
+
+   /* shutdown */
+   if (pfd[1].revents != 0) break;
  
    /* accept connection */
    connected_socket = SK_accept_connection(listening_socket);
@@ -497,7 +532,7 @@ do { /* be alive while do_server is 1. do_server is turned off by SIGINT */
     SK_cd_puts(&condat, buff);
     SK_cd_close(&(condat));
     
-    free(hostaddress);
+    UT_free(hostaddress);
     /* start the next loop */
     continue;
    }
@@ -553,11 +588,11 @@ do { /* be alive while do_server is 1. do_server is turned off by SIGINT */
 /*   fclose(ud_stream.log.logfile); */
  /* That's all. Close connection to the DB */ 
  SQ_close_connection(ud_stream.db_connection);
- free(db_host);
- free(db_name);
- free(db_user);
- free(db_passwd);
- free(source_name);
+ UT_free(db_host);
+ UT_free(db_name);
+ UT_free(db_user);
+ UT_free(db_passwd);
+ UT_free(source_name);
 
 
  ER_inf_va(FAC_UD, ASP_UD_UPDLOG, "%s update server stopped", UD_TAG);
